@@ -1,5 +1,5 @@
 """
-Assgnopts Assgn 1.3.0 CSV
+Assgnopts Assgn 1.3.5 CSV BUG FIXES
 
 Made with love by Albert Calasanz Sallen
 TODO:
@@ -8,7 +8,7 @@ Load CSV
 import os
 import csv
 import string
-import random, pandas
+import random
 os.system("")
 def List(list,load):                                     #Converteix un array en un string
     "Converts array to string"
@@ -128,7 +128,7 @@ class Assgn():
         self.Allow0 = False
         self.AllowStr = False
         self.OnlyInt = False
-        self.name = name
+        self.name =  name
         self.vals = False
         self.value = []
         self.title = ui
@@ -136,69 +136,72 @@ class Assgn():
         self.ln = None
         self.ans = None
         self.rules = rules
-        if type(vals) in (tuple,range,list):
-            self.vals = vals
-        elif vals is not None:
-            mess.InvalidInput()
-     # Comprovar les rules
-        self.conj = "" if type(conj) is not str else conj
-        if type(load) is list:
-            dic = dict.fromkeys(range(len(load)),"")
-            for x in dic:
-                dic[x] = load[x]
-            load = dic
-        elif type(load) is range:
-            self.ln = True
-        if rules is None:
-            pass
-        elif (type(rules) is list) and (len(rules) == 3):
-            if all(i in(True,False) for i in rules):
-                self.Allow0 = rules[0]
-                self.AllowNegative = rules[1]
-                self.OnlyInt = rules[2]
-            else:
-                print("Error on rules")
-                mess.Valerr()
-        if self.OnlyInt and not self.Allow0 and not self.AllowNegative:
-            self.rules = None
-        elif rules == "str":
-            self.AllowStr = True
-        else:
-            pass
-            #print(color.b.red+"Error in rules input\nAll will become false"+color.end)
+        self.conj = conj
+        self.rang = None
+        if not(load is None) and (rang is None):
+            if type(vals) in (tuple,range,list):
+                self.vals = vals
+            elif vals is not None:
+                mess.InvalidInput()
         # Comprovar les rules
-        self.ln = False if self.ln == None else True
-        if load is None: self.ln = True
-        if not(type(load) is dict) and not(self.ln):                                 #Verificar si load is dict
-            print("Line 121")
-            mess.DictErr()
-        else:
-            self.load = load
-            if type(load) is dict:
-                for j in list(self.load):
-                    if not(type(self.load[j]) in (tuple,list)):
-                        self.load[j] = (self.load[j],"")
-                #print(self.load)
-        #---------------------------------------------------------------------------------------
-        if isinstance(rang,range):
-            if not(len([x for x in rang]) > 0):
-                mess.Valerr()
+            self.conj = "" if type(conj) is not str else conj
+            if type(load) is list:
+                dic = dict.fromkeys(range(len(load)),"")
+                for x in dic:
+                    dic[x] = load[x]
+                load = dic
+            elif type(load) is range:
+                self.ln = True
+            if rules is None:
+                pass
+            elif (type(rules) is list) and (len(rules) == 3):
+                if all(i in(True,False) for i in rules):
+                    self.Allow0 = rules[0]
+                    self.AllowNegative = rules[1]
+                    self.OnlyInt = rules[2]
+                else:
+                    print("Error on rules")
+                    mess.Valerr()
+            if self.OnlyInt and not self.Allow0 and not self.AllowNegative:
+                self.rules = None
+            elif rules == "str":
+                self.AllowStr = True
+            else:
+                pass
+                #print(color.b.red+"Error in rules input\nAll will become false"+color.end)
+            # Comprovar les rules
+            self.ln = False if self.ln == None else True
+            if load is None: self.ln = True
+            if not(type(load) is dict) and not(self.ln):                                 #Verificar si load is dict
+                print("Line 121")
+                mess.DictErr()
+            else:
+                self.load = load
+                if type(load) is dict:
+                    for j in list(self.load):
+                        if not(type(self.load[j]) in (tuple,list)):
+                            self.load[j] = (self.load[j],"")
+                    #print(self.load)
+            #---------------------------------------------------------------------------------------
+            if isinstance(rang,range):
+                if not(len([x for x in rang]) > 0):
+                    mess.Valerr()
+                else:
+                    self.rang = rang
+                if self.ln:
+                    self.array = ["" for x in self.rang] if self.AllowStr else [0 for x in self.rang]
+            elif rang is None:
+                try:
+                    assert len(load)>0
+                except:
+                    mess.Valerr()
+                self.rang = range(len(load))
+                self.array = [0 for x in self.rang]
             else:
-                self.rang = rang
-            if self.ln:
-                self.array = ["" for x in self.rang] if self.AllowStr else [0 for x in self.rang]
-        elif rang is None:
-            try:
-                assert len(load)>0
-            except:
                 mess.Valerr()
-            self.rang = range(len(load))
-            self.array = [0 for x in self.rang]
-        else:
-            mess.Valerr()
-        # self.input()
+            # self.input()
     def input(self):
-        self.name = [objname for objname, oid in globals().items() if id(oid)==id(self)][0] if self.name is None else self.name
+        self.name = id(self) if self.name is None else self.name
         # HEADER
         if self.title:
             print(color.t.OKGREEN+"Set of {} questions".format(len(self.array))+color.end)
@@ -220,7 +223,7 @@ class Assgn():
                         if not(self.ln):
                             self.value = [x[1] for x in Dic2List(self.load)]
                             b = self.value[i][1]
-                            a = self.value[i][0]
+                            a = self.value[i][0]                                 #[x[0] for x in Dic2List(self.load)][i]
                         if len(self.conj) > 0 and len(b) > 0: magn = " "+self.conj+" "+str(b)+" : "
                         if self.AllowStr:
                             ans = str(input(self.no+"Enter "+str(a)+magn))
@@ -338,8 +341,11 @@ class Assgn():
     def clear(self):
         self.array = [0 for x in self.rang]
     def save(self,filename=None,rndm=False):
-        self.value = [x[1] for x in Dic2List(self.load)]
-        it = self.name
+        if not self.ln:
+            self.value = [x[1] for x in Dic2List(self.load)]
+        else:
+            self.value = [x[1] for x in Dic2List(List2Dict([x for x in self.load]))]
+        it = self.name if self.name is not None else str(id(self))
         if not filename:
             if rndm:
                 filename = id_generator() + "-" + it +".assgnopts"+ ".csv"
@@ -349,13 +355,16 @@ class Assgn():
         with open(filename, mode='w',newline='') as csv_file:
             fieldnames = ["DATA"]+[i[0] for i in self.value]
             writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
-            b = [x[0] for x in Dic2List(self.load)]
-            dic = Ar4Ar2Dic(fieldnames,[it]+[(b[idx],self.value[idx][1]) for idx,i in enumerate(b)])
+            if not self.ln:
+                b = [x[0] for x in Dic2List(self.load)]
+                dic = Ar4Ar2Dic(fieldnames,[it]+[(b[idx],self.value[idx][1]) for idx,i in enumerate(b)])
+            else:
+                b = [x[0] for x in Dic2List(List2Dict([x for x in self.load]))]
+                dic = Ar4Ar2Dic(fieldnames,[it]+[self.array[idx] for idx,i in enumerate(b)])
             writer.writeheader()
             writer.writerow(dic)
         return filename
     def loadcsv(self,file:str):
-        self.value = [x[1] for x in Dic2List(self.load)]
         with open(file, mode='r') as csv_file:
             csv_reader = csv.DictReader(csv_file)
             line_count = 0
@@ -372,8 +381,12 @@ class Assgn():
                         else:
                             self.array.insert(idx,x.replace("(","").replace(")","").split(", ")[0])
                         x = eval(x)
-                        dic[idx] = x
-                    self.load = dic
+                        dic[list(zip(row))[idx][0]] = x
+                    self.rang = range(len(dic))
+                    self.value = [x[1] for x in Dic2List(dic)]
+                    self.load = {}
+                    for idx,x in enumerate(Dic2List(dic)):
+                        self.load[idx] = (x[0],x[1][1])
                     line_count += 1
                 line_count += 1
     def __str__(self):
@@ -387,8 +400,8 @@ if __name__ == "__main__":
         0: ("Peres","kilos"),
         1: ("Pomes","unitats")
     }
-    #ElMeuObjecte = Assgn(dic)
-    #print(ElMeuObjecte)
+    """ ElMeuObjecte = Assgn(dic)
+    print(ElMeuObjecte) """
     """ MyObject = Assgn(Ar2Dict(IterAr(1,"Object"),"units"),conj="as",vals=range(1,10))
     MySecondObject = Assgn(dic)
     print("________________________________________")
@@ -400,7 +413,13 @@ if __name__ == "__main__":
     MyThirdObject.input()
     print(MyThirdObject.getValues())
     MyThirdObject.dispValues() """
-    data = Assgn(Ar2Dict(IterAr(5,"label"),"ACU(s)"),name="data")
+    """ data = Assgn(Ar2Dict(IterAr(5,"label"),"ACU(s)"))
     datafile = data.save()
     print(pandas.read_csv(datafile,header=0,index_col='DATA'),end="\n\n")
-    data.loadcsv(datafile)
\ No newline at end of file
+    data.loadcsv(datafile) """
+    """ data = Assgn(conj="as")
+    data.loadcsv("data.assgnopts.csv")
+    data.input() """
+    data = Assgn(Ar2Dict(IterAr(5,"peres"),"kilos"))
+    data.input()
+    data.save()
\ No newline at end of file
